/*
 * WARNING: do not edit!
 * Generated by Makefile from providers/common/der/der_slh_dsa_gen.c.in
 *
 * Copyright 2025 The OpenSSL Project Authors. All Rights Reserved.
 *
 * Licensed under the Apache License 2.0 (the "License").  You may not use
 * this file except in compliance with the License.  You can obtain a copy
 * in the file LICENSE in the source distribution or at
 * https://www.openssl.org/source/license.html
 */

#include "prov/der_slh_dsa.h"

/* Well known OIDs precompiled */
/* clang-format off */

/*
 * id-slh-dsa-sha2-128s OBJECT IDENTIFIER ::= { sigAlgs 20 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_sha2_128s[DER_OID_SZ_id_slh_dsa_sha2_128s] = {
    DER_OID_V_id_slh_dsa_sha2_128s
};

/*
 * id-slh-dsa-sha2-128f OBJECT IDENTIFIER ::= { sigAlgs 21 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_sha2_128f[DER_OID_SZ_id_slh_dsa_sha2_128f] = {
    DER_OID_V_id_slh_dsa_sha2_128f
};

/*
 * id-slh-dsa-sha2-192s OBJECT IDENTIFIER ::= { sigAlgs 22 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_sha2_192s[DER_OID_SZ_id_slh_dsa_sha2_192s] = {
    DER_OID_V_id_slh_dsa_sha2_192s
};

/*
 * id-slh-dsa-sha2-192f OBJECT IDENTIFIER ::= { sigAlgs 23 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_sha2_192f[DER_OID_SZ_id_slh_dsa_sha2_192f] = {
    DER_OID_V_id_slh_dsa_sha2_192f
};

/*
 * id-slh-dsa-sha2-256s OBJECT IDENTIFIER ::= { sigAlgs 24 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_sha2_256s[DER_OID_SZ_id_slh_dsa_sha2_256s] = {
    DER_OID_V_id_slh_dsa_sha2_256s
};

/*
 * id-slh-dsa-sha2-256f OBJECT IDENTIFIER ::= { sigAlgs 25 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_sha2_256f[DER_OID_SZ_id_slh_dsa_sha2_256f] = {
    DER_OID_V_id_slh_dsa_sha2_256f
};

/*
 * id-slh-dsa-shake-128s OBJECT IDENTIFIER ::= { sigAlgs 26 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_shake_128s[DER_OID_SZ_id_slh_dsa_shake_128s] = {
    DER_OID_V_id_slh_dsa_shake_128s
};

/*
 * id-slh-dsa-shake-128f OBJECT IDENTIFIER ::= { sigAlgs 27 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_shake_128f[DER_OID_SZ_id_slh_dsa_shake_128f] = {
    DER_OID_V_id_slh_dsa_shake_128f
};

/*
 * id-slh-dsa-shake-192s OBJECT IDENTIFIER ::= { sigAlgs 28 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_shake_192s[DER_OID_SZ_id_slh_dsa_shake_192s] = {
    DER_OID_V_id_slh_dsa_shake_192s
};

/*
 * id-slh-dsa-shake-192f OBJECT IDENTIFIER ::= { sigAlgs 29 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_shake_192f[DER_OID_SZ_id_slh_dsa_shake_192f] = {
    DER_OID_V_id_slh_dsa_shake_192f
};

/*
 * id-slh-dsa-shake-256s OBJECT IDENTIFIER ::= { sigAlgs 30 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_shake_256s[DER_OID_SZ_id_slh_dsa_shake_256s] = {
    DER_OID_V_id_slh_dsa_shake_256s
};

/*
 * id-slh-dsa-shake-256f OBJECT IDENTIFIER ::= { sigAlgs 31 }
 */
const unsigned char ossl_der_oid_id_slh_dsa_shake_256f[DER_OID_SZ_id_slh_dsa_shake_256f] = {
    DER_OID_V_id_slh_dsa_shake_256f
};

/* clang-format on */
