/*
 * WARNING: do not edit!
 * Generated by Makefile from providers/common/der/der_hkdf_gen.c.in
 *
 * Copyright 2025 The OpenSSL Project Authors. All Rights Reserved.
 *
 * Licensed under the Apache License 2.0 (the "License").  You may not use
 * this file except in compliance with the License.  You can obtain a copy
 * in the file LICENSE in the source distribution or at
 * https://www.openssl.org/source/license.html
 */

#include "prov/der_hkdf.h"

/* Well known OIDs precompiled */
/* clang-format off */

/*
 * id-alg-hkdf-with-sha256 OBJECT IDENTIFIER ::= { id-alg 28 }
 */
const unsigned char ossl_der_oid_id_alg_hkdf_with_sha256[DER_OID_SZ_id_alg_hkdf_with_sha256] = {
    DER_OID_V_id_alg_hkdf_with_sha256
};

/*
 * id-alg-hkdf-with-sha384 OBJECT IDENTIFIER ::= { id-alg 29 }
 */
const unsigned char ossl_der_oid_id_alg_hkdf_with_sha384[DER_OID_SZ_id_alg_hkdf_with_sha384] = {
    DER_OID_V_id_alg_hkdf_with_sha384
};

/*
 * id-alg-hkdf-with-sha512 OBJECT IDENTIFIER ::= { id-alg 30 }
 */
const unsigned char ossl_der_oid_id_alg_hkdf_with_sha512[DER_OID_SZ_id_alg_hkdf_with_sha512] = {
    DER_OID_V_id_alg_hkdf_with_sha512
};

/* clang-format on */
