/*
 * WARNING: do not edit!
 * Generated by Makefile from providers/common/der/der_ml_dsa_gen.c.in
 *
 * Copyright 2025 The OpenSSL Project Authors. All Rights Reserved.
 *
 * Licensed under the Apache License 2.0 (the "License").  You may not use
 * this file except in compliance with the License.  You can obtain a copy
 * in the file LICENSE in the source distribution or at
 * https://www.openssl.org/source/license.html
 */

#include "prov/der_ml_dsa.h"

/* Well known OIDs precompiled */
/* clang-format off */

/*
 * id-ml-dsa-44 OBJECT IDENTIFIER ::= { sigAlgs 17 }
 */
const unsigned char ossl_der_oid_id_ml_dsa_44[DER_OID_SZ_id_ml_dsa_44] = {
    DER_OID_V_id_ml_dsa_44
};

/*
 * id-ml-dsa-65 OBJECT IDENTIFIER ::= { sigAlgs 18 }
 */
const unsigned char ossl_der_oid_id_ml_dsa_65[DER_OID_SZ_id_ml_dsa_65] = {
    DER_OID_V_id_ml_dsa_65
};

/*
 * id-ml-dsa-87 OBJECT IDENTIFIER ::= { sigAlgs 19 }
 */
const unsigned char ossl_der_oid_id_ml_dsa_87[DER_OID_SZ_id_ml_dsa_87] = {
    DER_OID_V_id_ml_dsa_87
};

/* clang-format on */
